@@ -43,7 +43,7 @@ class TQSLStationListBox;
 class TQSLGetStationNameDialog : public wxDialog {
  public:
 	TQSLGetStationNameDialog(wxWindow *parent, wxHtmlHelpController *help = 0, const wxPoint& pos = wxDefaultPosition,
-		bool i_issave = false, const wxString& title = wxT(""), const wxString& okLabel = wxT("OK"), bool i_editonly = false);
+		bool i_issave = false, const wxString& title = wxT(""), const wxString& okLabel = _("OK"), bool i_editonly = false);
 	void OnOk(wxCommandEvent& event);
 	void OnCancel(wxCommandEvent& event);
 	void OnDelete(wxCommandEvent& event);
