@@ -22,6 +22,12 @@
     #include <direct.h>
     #include <Shlobj.h>
 #endif
+#ifdef __APPLE__
+#include <CoreFoundation/CFBundle.h>
+#endif
+#include <string>
+using std::string;
+
 #include <openssl/err.h>
 #include <openssl/objects.h>
 #include <openssl/evp.h>
@@ -40,6 +46,7 @@ DLLEXPORTDATA int tQSL_Error = 0;
 DLLEXPORTDATA int tQSL_Errno = 0;
 DLLEXPORTDATA TQSL_ADIF_GET_FIELD_ERROR tQSL_ADIF_Error;
 DLLEXPORTDATA const char *tQSL_BaseDir = 0;
+DLLEXPORTDATA const char *tQSL_RsrcDir = 0;
 DLLEXPORTDATA char tQSL_ErrorFile[256];
 DLLEXPORTDATA char tQSL_CustomError[256];
 DLLEXPORTDATA char tQSL_ImportCall[256];
@@ -98,7 +105,8 @@ static const char *error_strings[] = {
 	"Certificate provider not found",			/* TQSL_PROVIDER_NOT_FOUND */
 	"No callsign certificate for key",			/* TQSL_CERT_KEY_ONLY */
 	"Configuration file cannot be opened",			/* TQSL_CONFIG_ERROR */
-	"Callsign Certificate or Certificate Request not found",/* TQSL_CERT_NOT_FOUND */
+	"The private key for this Callsign Certificate is not present on this computer; you can obtain it by loading a .tbk or .p12 file",
+								/* TQSL_CERT_NOT_FOUND */
 	"PKCS#12 file not TQSL compatible",			/* TQSL_PKCS12_ERROR */
 	"Callsign Certificate not TQSL compatible",		/* TQSL_CERT_TYPE_ERROR */
 	"Date out of range",					/* TQSL_DATE_OUT_OF_RANGE */
@@ -175,6 +183,59 @@ static int pmkdir(const char *path, int perm) {
 }
 #endif // defined(_WIN32)
 
+static void
+tqsl_get_rsrc_dir() {
+	tqslTrace("tqsl_get_rsrc_dir", NULL);
+
+#ifdef _WIN32
+	HKEY hkey;
+	DWORD dtype;
+	char wpath[TQSL_MAX_PATH_LEN];
+	DWORD bsize = sizeof wpath;
+	int wval;
+	if ((wval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+		"Software\\TrustedQSL", 0, KEY_READ, &hkey)) == ERROR_SUCCESS) {
+		wval = RegQueryValueEx(hkey, "InstallPath", 0, &dtype, (LPBYTE)wpath, &bsize);
+		RegCloseKey(hkey);
+		if (wval == ERROR_SUCCESS) {
+			string p = string(wpath);
+			if (p[p.length() -1] == '\\')
+				p = p.substr(0, p.length() - 1);
+			tQSL_RsrcDir = strdup(p.c_str());
+		}
+	}
+#elif defined(__APPLE__)
+	// Get path to config.xml resource from bundle
+	CFBundleRef tqslBundle = CFBundleGetMainBundle();
+	CFURLRef configXMLURL = CFBundleCopyResourceURL(tqslBundle, CFSTR("config"), CFSTR("xml"), NULL);
+	if (configXMLURL) {
+		CFStringRef pathString = CFURLCopyFileSystemPath(configXMLURL, kCFURLPOSIXPathStyle);
+		CFRelease(configXMLURL);
+
+		// Convert CFString path to config.xml to string object
+		CFIndex maxStringLengthInBytes = CFStringGetMaximumSizeForEncoding(CFStringGetLength(pathString), kCFStringEncodingUTF8);
+		char *pathCString = static_cast<char *>(malloc(maxStringLengthInBytes));
+		if (pathCString) {
+			CFStringGetCString(pathString, pathCString, maxStringLengthInBytes, kCFStringEncodingASCII);
+			CFRelease(pathString);
+			size_t p;
+			string path = string(pathCString);
+			if ((p = path.find("/config.xml")) != string::npos) {
+				path = path.substr(0, p);
+			}
+			tQSL_RsrcDir = strdup(path.c_str());
+			free(pathCString);
+		}
+	}
+#else
+	string p = CONFDIR;
+	if (p.back() == '/')
+		p.pop_back();
+	tQSL_RsrcDir = strdup(p.c_str());
+#endif
+	tqslTrace("tqsl_get_rsrc_dir", "rsrc_path=%s", tQSL_RsrcDir);
+}
+
 DLLEXPORT int CALLCONVENTION
 tqsl_init() {
 	static char semaphore = 0;
@@ -232,6 +293,9 @@ tqsl_init() {
 			return 1;
 		}
 	}
+	if (tQSL_RsrcDir == NULL) {
+		tqsl_get_rsrc_dir();
+	}
 	if (tQSL_BaseDir == NULL) {
 #if defined(_WIN32)
 		wchar_t *wcp;
@@ -392,7 +456,7 @@ tqsl_getErrorString_v(int err) {
 	}
 	if (err == TQSL_CERT_NOT_FOUND && tQSL_ImportCall[0] != '\0') {
 		snprintf(buf, sizeof buf,
-			"Callsign Certificate or Certificate Request not found for callsign %s serial %ld",
+			"The private key for callsign %s serial %ld is not present on this computer; you can obtain it by loading a .tbk or .p12 file",
 			tQSL_ImportCall, tQSL_ImportSerial);
 		tQSL_ImportCall[0] = '\0';
 		return buf;
